@@ -1,27 +1,37 @@
-/* $Id: SUPDrv-dtrace.cpp 135976 2020-02-04 10:35:17Z bird $ */
+/* $Id: SUPDrv-dtrace.cpp 153224 2022-08-22 17:43:14Z klaus $ */
 /** @file
  * VBoxDrv - The VirtualBox Support Driver - DTrace Provider.
  */
 
 /*
- * Copyright (C) 2012-2020 Oracle Corporation
+ * Copyright (C) 2012-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -88,7 +98,7 @@
 #  include <dtrace/enabling.h> /* Missing from provider.h. */
 #  include <dtrace/arg.h> /* Missing from provider.h. */
 # endif
-# include <linux/kallsyms.h>
+# include <linux/module.h>
 /** Status code fixer (UEK uses linux convension unlike the others). */
 # define FIX_UEK_RC(a_rc) (-(a_rc))
 #else
@@ -236,13 +246,17 @@ typedef VBDTSTACKDATA *PVBDTSTACKDATA;
 #if defined(RT_OS_DARWIN) || defined(RT_OS_LINUX)
 /** @name DTrace kernel interface used on Darwin and Linux.
  * @{ */
-static void        (* g_pfnDTraceProbeFire)(dtrace_id_t, uint64_t, uint64_t, uint64_t, uint64_t, uint64_t);
-static dtrace_id_t (* g_pfnDTraceProbeCreate)(dtrace_provider_id_t, const char *, const char *, const char *, int, void *);
-static dtrace_id_t (* g_pfnDTraceProbeLookup)(dtrace_provider_id_t, const char *, const char *, const char *);
-static int         (* g_pfnDTraceProviderRegister)(const char *, const dtrace_pattr_t *, uint32_t, /*cred_t*/ void *,
-                                                   const dtrace_pops_t *, void *, dtrace_provider_id_t *);
-static void        (* g_pfnDTraceProviderInvalidate)(dtrace_provider_id_t);
-static int         (* g_pfnDTraceProviderUnregister)(dtrace_provider_id_t);
+static DECLCALLBACKPTR_EX(void,        RT_NOTHING, g_pfnDTraceProbeFire,(dtrace_id_t, uint64_t, uint64_t, uint64_t, uint64_t,
+                                                                         uint64_t));
+static DECLCALLBACKPTR_EX(dtrace_id_t, RT_NOTHING, g_pfnDTraceProbeCreate,(dtrace_provider_id_t, const char *, const char *,
+                                                                           const char *, int, void *));
+static DECLCALLBACKPTR_EX(dtrace_id_t, RT_NOTHING, g_pfnDTraceProbeLookup,(dtrace_provider_id_t, const char *, const char *,
+                                                                           const char *));
+static DECLCALLBACKPTR_EX(int,         RT_NOTHING, g_pfnDTraceProviderRegister,(const char *, const dtrace_pattr_t *, uint32_t,
+                                                                                /*cred_t*/ void *, const dtrace_pops_t *,
+                                                                                void *, dtrace_provider_id_t *));
+static DECLCALLBACKPTR_EX(void,        RT_NOTHING, g_pfnDTraceProviderInvalidate,(dtrace_provider_id_t));
+static DECLCALLBACKPTR_EX(int,         RT_NOTHING, g_pfnDTraceProviderUnregister,(dtrace_provider_id_t));
 
 #define dtrace_probe            g_pfnDTraceProbeFire
 #define dtrace_probe_create     g_pfnDTraceProbeCreate
@@ -251,6 +265,21 @@ static int         (* g_pfnDTraceProviderUnregister)(dtrace_provider_id_t);
 #define dtrace_invalidate       g_pfnDTraceProviderInvalidate
 #define dtrace_unregister       g_pfnDTraceProviderUnregister
 
+/** For dynamical resolving and releasing.   */
+static const struct
+{
+    const char *pszName;
+    uintptr_t  *ppfn; /**< @note Clang 11 nothrow weirdness forced this from PFNRT * to uintptr_t *. */
+} g_aDTraceFunctions[] =
+{
+    { "dtrace_probe",        (uintptr_t *)&dtrace_probe        },
+    { "dtrace_probe_create", (uintptr_t *)&dtrace_probe_create },
+    { "dtrace_probe_lookup", (uintptr_t *)&dtrace_probe_lookup },
+    { "dtrace_register",     (uintptr_t *)&dtrace_register     },
+    { "dtrace_invalidate",   (uintptr_t *)&dtrace_invalidate   },
+    { "dtrace_unregister",   (uintptr_t *)&dtrace_unregister   },
+};
+
 /** @} */
 #endif
 
@@ -1107,53 +1136,60 @@ const SUPDRVTRACERREG * VBOXCALL supdrvDTraceInit(void)
     }
 # endif
 
-    static const struct
-    {
-        const char *pszName;
-        PFNRT      *ppfn;
-    } s_aDTraceFunctions[] =
-    {
-        { "dtrace_probe",        (PFNRT*)&dtrace_probe        },
-        { "dtrace_probe_create", (PFNRT*)&dtrace_probe_create },
-        { "dtrace_probe_lookup", (PFNRT*)&dtrace_probe_lookup },
-        { "dtrace_register",     (PFNRT*)&dtrace_register     },
-        { "dtrace_invalidate",   (PFNRT*)&dtrace_invalidate   },
-        { "dtrace_unregister",   (PFNRT*)&dtrace_unregister   },
-    };
     unsigned i;
-    for (i = 0; i < RT_ELEMENTS(s_aDTraceFunctions); i++)
+    for (i = 0; i < RT_ELEMENTS(g_aDTraceFunctions); i++)
     {
 # ifndef RT_OS_LINUX
-        rc = RTR0DbgKrnlInfoQuerySymbol(hKrnlInfo, NULL, s_aDTraceFunctions[i].pszName,
-                                        (void **)s_aDTraceFunctions[i].ppfn);
+        rc = RTR0DbgKrnlInfoQuerySymbol(hKrnlInfo, NULL, g_aDTraceFunctions[i].pszName,
+                                        (void **)g_aDTraceFunctions[i].ppfn);
         if (RT_FAILURE(rc))
         {
-            SUPR0Printf("supdrvDTraceInit: Failed to resolved '%s' (rc=%Rrc, i=%u).\n", s_aDTraceFunctions[i].pszName, rc, i);
+            SUPR0Printf("supdrvDTraceInit: Failed to resolved '%s' (rc=%Rrc, i=%u).\n", g_aDTraceFunctions[i].pszName, rc, i);
             break;
         }
-# else
-        unsigned long ulAddr = kallsyms_lookup_name(s_aDTraceFunctions[i].pszName);
+# else /* RT_OS_LINUX */
+        uintptr_t ulAddr = (uintptr_t)__symbol_get(g_aDTraceFunctions[i].pszName);
         if (!ulAddr)
         {
-            SUPR0Printf("supdrvDTraceInit: Failed to resolved '%s' (i=%u).\n", s_aDTraceFunctions[i].pszName, i);
+            SUPR0Printf("supdrvDTraceInit: Failed to resolved '%s' (i=%u).\n", g_aDTraceFunctions[i].pszName, i);
+            while (i-- > 0)
+            {
+                __symbol_put(g_aDTraceFunctions[i].pszName);
+                *g_aDTraceFunctions[i].ppfn = NULL;
+            }
             return NULL;
         }
-        *s_aDTraceFunctions[i].ppfn = (PFNRT)ulAddr;
-# endif
+        *g_aDTraceFunctions[i].ppfn = (PFNRT)ulAddr;
+# endif /* RT_OS_LINUX */
     }
 
 # ifndef RT_OS_LINUX
     RTR0DbgKrnlInfoRelease(hKrnlInfo);
     if (RT_FAILURE(rc))
         return NULL;
-# else
-    /** @todo grab a reference to the dtrace module... */
 # endif
 #endif
 
     return &g_VBoxDTraceReg;
 }
 
+/**
+ * Module teardown code.
+ */
+void VBOXCALL supdrvDTraceFini(void)
+{
+#ifdef RT_OS_LINUX
+    /* Release the references. */
+    unsigned i;
+    for (i = 0; i < RT_ELEMENTS(g_aDTraceFunctions); i++)
+        if (*g_aDTraceFunctions[i].ppfn)
+        {
+            __symbol_put(g_aDTraceFunctions[i].pszName);
+            *g_aDTraceFunctions[i].ppfn = NULL;
+        }
+#endif
+}
+
 #ifndef VBOX_WITH_NATIVE_DTRACE
 # error "VBOX_WITH_NATIVE_DTRACE is not defined as it should"
 #endif
