@@ -1,31 +1,44 @@
-/* $Id: iprt.h 118412 2017-10-17 14:26:02Z bird $ */
+/* $Id: iprt.h 153224 2022-08-22 17:43:14Z klaus $ */
 /** @file
  * IPRT - Internal header for miscellaneous global defs and types.
  */
 
 /*
- * Copyright (C) 2009-2017 Oracle Corporation
+ * Copyright (C) 2009-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
-#ifndef ___internal_iprt_h
-#define ___internal_iprt_h
+#ifndef IPRT_INCLUDED_INTERNAL_iprt_h
+#define IPRT_INCLUDED_INTERNAL_iprt_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/cdefs.h>
 #include <iprt/types.h>
@@ -60,6 +73,47 @@
 # define RT_EXPORT_SYMBOL(Name) extern int g_rtExportSymbolDummyVariable
 #endif
 
+/** @def RT_ALIAS_AND_EXPORT_NOCRT_SYMBOL
+ * Creates symbol alias and export a RT_NOCRT symbol.
+ *
+ * For targets using ELF this macro generates weak symbol aliases, for the other
+ * object formats the makefile scans source files for this macro and produces
+ * separate object files with the aliases.
+ *
+ * @param   a_Name      The CRT function or variable name.
+ *
+ * @todo    Does not support ELF targets using underscore prefixed symbols.
+ */
+#if defined(__ELF__) && defined(RT_WITH_NOCRT_ALIASES) && !defined(RT_WITHOUT_NOCRT_ALIASES)
+# ifdef RT_WITH_NOCRT_UNDERSCORE_ALIASES
+#  define RT_ALIAS_AND_EXPORT_NOCRT_SYMBOL(a_Name) RT_EXPORT_SYMBOL(a_Name) \
+    __asm__(".weak " #a_Name "\n\t" \
+            ".set  " #a_Name "," RT_NOCRT_STR(a_Name) "\n\t" \
+            ".weak _" #a_Name "\n\t" \
+            ".set  _" #a_Name "," RT_NOCRT_STR(a_Name) "\n\t")
+# else
+#  define RT_ALIAS_AND_EXPORT_NOCRT_SYMBOL(a_Name) RT_EXPORT_SYMBOL(a_Name) \
+    __asm__(".weak " #a_Name "\n\t" \
+            ".set  " #a_Name "," RT_NOCRT_STR(a_Name) "\n\t")
+# endif
+#else
+# define RT_ALIAS_AND_EXPORT_NOCRT_SYMBOL(a_Name) RT_EXPORT_SYMBOL(a_Name)
+#endif
+
+/** @def RT_ALIAS_AND_EXPORT_NOCRT_SYMBOL_WITHOUT_UNDERSCORE
+ * Variant of RT_ALIAS_AND_EXPORT_NOCRT_SYMBOL that omits the
+ * underscore-prefixed variant of the symbol.
+ *
+ * @param   a_Name      The CRT function or variable name.
+ */
+#if defined(__ELF__) && defined(RT_WITH_NOCRT_ALIASES) && !defined(RT_WITHOUT_NOCRT_ALIASES)
+# define RT_ALIAS_AND_EXPORT_NOCRT_SYMBOL_WITHOUT_UNDERSCORE(a_Name) RT_EXPORT_SYMBOL(a_Name) \
+    __asm__(".weak " #a_Name "\n\t" \
+            ".set  " #a_Name "," RT_NOCRT_STR(a_Name) "\n\t")
+#else
+# define RT_ALIAS_AND_EXPORT_NOCRT_SYMBOL_WITHOUT_UNDERSCORE(a_Name) RT_EXPORT_SYMBOL(a_Name)
+#endif
+
 
 /** @def RT_MORE_STRICT
  * Enables more assertions in IPRT. */
@@ -200,5 +254,5 @@ uint32_t rtR0SemWaitOs2ConvertTimeout(uint32_t fFlags, uint64_t uTimeout);
 
 RT_C_DECLS_END
 
-#endif
+#endif /* !IPRT_INCLUDED_INTERNAL_iprt_h */
 
