@@ -1,27 +1,37 @@
-/* $Id: thread.h 136258 2020-02-21 22:58:42Z bird $ */
+/* $Id: thread.h 153224 2022-08-22 17:43:14Z klaus $ */
 /** @file
  * IPRT - Internal RTThread header.
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 #ifndef IPRT_INCLUDED_INTERNAL_thread_h
@@ -42,12 +52,13 @@
 #ifdef RT_WITH_ICONV_CACHE
 # include "internal/string.h"
 #endif
+#if defined(IPRT_NO_CRT) && defined(IN_RING3)
+# include "internal/nocrt.h"
+#endif
 
 RT_C_DECLS_BEGIN
 
 
-/** Max thread name length. */
-#define RTTHREAD_NAME_LEN       16
 #ifdef IPRT_WITH_GENERIC_TLS
 /** The number of TLS entries for the generic implementation. */
 # define RTTHREAD_TLS_ENTRIES   64
@@ -107,7 +118,7 @@ typedef struct RTTHREADINT
 #ifdef RT_WITH_ICONV_CACHE
     /** Handle cache for iconv.
      * @remarks ASSUMES sizeof(void *) >= sizeof(iconv_t). */
-    void *ahIconvs[RTSTRICONV_END];
+    void                   *ahIconvs[RTSTRICONV_END];
 #endif
 #ifdef IPRT_WITH_GENERIC_TLS
     /** The TLS entries for this thread. */
@@ -115,6 +126,10 @@ typedef struct RTTHREADINT
 #endif
     /** Thread name. */
     char                    szName[RTTHREAD_NAME_LEN];
+#if defined(IPRT_NO_CRT) && defined(IN_RING3)
+    /** No-CRT per thread data. */
+    RTNOCRTTHREADDATA       NoCrt;
+#endif
 } RTTHREADINT;
 /** Pointer to the internal representation of a thread. */
 typedef RTTHREADINT *PRTTHREADINT;
@@ -137,7 +152,7 @@ typedef RTTHREADINT *PRTTHREADINT;
 /** @} */
 
 /** Counters for each thread type. */
-extern DECLHIDDEN(uint32_t volatile)   g_acRTThreadTypeStats[RTTHREADTYPE_END];
+extern DECL_HIDDEN_DATA(uint32_t volatile) g_acRTThreadTypeStats[RTTHREADTYPE_END];
 
 
 /**
@@ -245,7 +260,7 @@ DECLHIDDEN(void) rtThreadNativeInformDebugger(PRTTHREADINT pThread);
 
 
 /* thread.cpp */
-DECLCALLBACK(DECLHIDDEN(int)) rtThreadMain(PRTTHREADINT pThread, RTNATIVETHREAD NativeThread, const char *pszThreadName);
+DECL_HIDDEN_CALLBACK(int) rtThreadMain(PRTTHREADINT pThread, RTNATIVETHREAD NativeThread, const char *pszThreadName);
 DECLHIDDEN(uint32_t)     rtThreadRelease(PRTTHREADINT pThread);
 DECLHIDDEN(void)         rtThreadTerminate(PRTTHREADINT pThread, int rc);
 DECLHIDDEN(PRTTHREADINT) rtThreadGetByNative(RTNATIVETHREAD NativeThread);
