@@ -1,27 +1,37 @@
-/* $Id: VBoxNetAdp-linux.c 149586 2022-01-28 09:26:21Z vgalitsy $ */
+/* $Id: VBoxNetAdp-linux.c 153224 2022-08-22 17:43:14Z klaus $ */
 /** @file
  * VBoxNetAdp - Virtual Network Adapter Driver (Host), Linux Specific Code.
  */
 
 /*
- * Copyright (C) 2009-2020 Oracle Corporation
+ * Copyright (C) 2009-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -282,7 +292,12 @@ static void vboxNetAdpNetDevInit(struct net_device *pNetDev)
     pNetDev->get_stats = vboxNetAdpLinuxGetStats;
 #endif /* RTLNX_VER_MAX(2,6,29) */
 #if RTLNX_VER_MIN(4,10,0)
-    pNetDev->max_mtu = 16110;
+    pNetDev->max_mtu = 65536;
+    pNetDev->features =   NETIF_F_TSO
+                        | NETIF_F_TSO6
+                        | NETIF_F_TSO_ECN
+                        | NETIF_F_SG
+                        | NETIF_F_HW_CSUM;
 #endif /* RTLNX_VER_MIN(4,10,0) */
 
     pNetDev->ethtool_ops = &gEthToolOpsVBoxNetAdp;
